from os.path import exists, isfile
from threading import Thread, Event, RLock
from ao import AudioDevice, aoError
from mad import MadFile

from Threading import Repeater

class Decoder(object):
  """
  This is our all-purpose decoder.  Pass it the driver name, and you're ready
  to start playing files:

  myD = Decoder("sun")
  myD.play("path/to/an.mp3")

  You will likely want to call play() in a thread, so you can have some control
  over pausing/stopping/seeking etc. while the song is playing.  This should be
  thread-safe. (it is for me .. so far! :)
  
  Functionality is mostly complete:

  action        function
  ------        --------
  play          play()
  pause         pause()
  resume        resume()
  toggle pause  togglePause()
  stop          stop()
  ffwd          seek(positive int)
  rewind        seek(negative int)

  grabDSP() and ungrabDSP() take and release control of the sound device,
  respctively.  This makes it simple to give up the sound device to other
  processes when needed, without having to exit, or delete the decoder.
  """
  
  def __init__(self, driver, rate=44100, bits=16):
    """
    Decoder(driver[, rate[, bits]]) -> Decoder object

    The only real action taken upon initialization of Decoder is grabbing and
    then immediately ungrabbing the sound device, so as to immediately
    determine any problem with it.  This actually may change, maybe we want
    to be able to create a Decoder while the sound device is being used.
    """

    self.driver = driver
    self.rate   = rate
    self.bits   = bits

    self.dsp = None

    # testing, testing, is this thing on?  hello?
    self.grabDSP()
    self.ungrabDSP()

  #

  def seek(self, position):
    "seek(position_in_seconds) -> None.  Goes to specified position in song."

    if self.stopped.isSet():
      return

    if position < 0:
      position = self.getSongLen() - abs(position)

    #try:
    self.madfile.seek_time(position*1000)
    #except:
      #return

  #

  getSongLen        = lambda self: self.madfile.total_time()/1000
  getSongBitrate    = lambda self: self.madfile.bitrate()/1000
  getSongSampleRate = lambda self: self.madfile.samplerate()

  #

  def stop(self):


  def pause(self):

  def resume(self):

  def togglePause(self):

  #

  def play(self, filename):
    if not filename or not isinstance(filename, str):
      raise AttributeError, "Need a proper filename, got `%s' instead" % filename
      return
    if not exists(filename):
      raise IOError, "`%s' does not exist" % filename
      return
    if not isfile(filename):
      raise IOError, "`%s' is not a regular file" % filename
      return




  def _play(self, filename):
    """
    play([filename]) -> None.  Play an mp3 file.

    If filename is empty or a non-string, AttributeError will be raised.
    """





class ThumpDecoder(Decoder):

  """
  ThumpDecoder is a slightly modified Decoder; it logs, takes care of the driver
  name for us [ and other undertermined stuff ].
  """

  debug = lambda self, msg: self.log and self.log.debug("ThumpDecoder: %s" %msg)

  def __init__(self, log=None):
    """
    ThumpDecoder([log]) -> ThumpDecoder object

    The optional log parameter should be a logger object from logger.getLogger()
    """

    self.log = log
    self.debug("__init__: running...")

    Decoder.__init__(self, "oss") # read from config

    self.debug("__init__: done")

  def wrap(self, function, name):
    "wrap(function, name) -> None.  Wraps a function with logging."
    self.debug("%s: running..." % name)
    function()
    self.debug("%s: done" % name)
  
