from os.path import exists

"""Generic file classes, derived from and made to be
more convenient than Python's built-in file class."""

class File(file):
  "Generic file class with convenient data access methods"
  
  def __init__(self, filename, mode="rw"):
    if not exists(filename): open(filename, "w").close()
    file.__init__(self, filename, mode)

  def reOpen(self, mode=None):
    """Close and re-open the file.  Optional $mode
    parameter specifies the mode to re-open in."""
    self.close()
    file.__init__(self.name, mode and mode or self.mode)

  def niceWriteLines(self, lines):
    "Works like writelines() but does all the newline nitpicking for you"
    return self.writelines( [ line.rstrip()+"\n" for line in lines ] )

  def tail(self, numLines=1):
    "Returns $numLines rstrip'ed lines from end of file"
    return [ line.rstrip() for line in self.readlines() ][-numLines:]


class NullFile(object):
  "A dummy file class that walks and talks like a file, but does nothing."

  mode = "rw"
  name = "/dev/null"
  closed = False
  softspace = False
  #encoding = ?

  close = fileno = flush = readinto = writelines = write = truncate = seek = \
    lambda *a, **b: None

  xreadlines = readline = read = lambda *a: str()

  isatty    = lambda: False
  tell      = lambda: int()
  readlines = lambda *a: list()

