import socket
from SocketServer import StreamRequestHandler, TCPServer, UnixStreamServer
from Queue import Queue
from threading import Thread


class MyHandler(StreamRequestHandler):
  def handle(self):
    line = self.rfile.readline(2048).strip()
    if line:
      self.server.inbox.put(line)


class SocketCommandCollector:

  msg = dict()

  def __init__(self, incoming, outgoing=None):
    """
    SocketCommandCollector(incoming, outgoing) -> SocketCommandCollector object

    Each parameter should be a tuple or list containing:
      for a unix domain socket: ("unix", "filename")
      for a tcp inet socket: ("tcp", ("ip.add.re.ss", port))

    SocketCommandCollector Starts up a SocketServer, and then expects to have
    its pop() method called continuously, which returns a dict with command and
    data keys.  Input is one command per line/connection, with an optional space
    and data after the space.  See the doc for parse() for more info about that.
    """

    if incoming[0] not in ("tcp", "unix") or \
       (outgoing and outgoing[0] not in ("tcp", "unix")):
      raise ValueError, "family should be either `tcp' or `unix'"

    self.incoming = incoming
    self.outgoing = outgoing

    if incoming[0] == "tcp":
      self.server = TCPServer(incoming[1], MyHandler)
    else:
      self.server = UnixStreamServer(incoming[1], MyHandler)

    self.server.inbox = Queue()

    thread = Thread(target=self.server.serve_forever)
    thread.setDaemon(True)
    thread.start()

  def size(self):
    return self.server.inbox.qsize()
  
  def update(self):
    pass

  def send(self, msg):
    if not self.outgoing:
      return

    proto = (self.outgoing[0] == "tcp") and socket.AF_INET or socket.AF_UNIX

    s = socket.socket(proto, socket.SOCK_STREAM)
    s.connect(self.outgoing[1])
    s.send(msg + "\r\n")
    s.close()

  def get(self):
    "get() -> msg.  Returns next message."
    return self.parse(self.server.inbox.get())

  def parse(self, text):
    r"""
    parse(text) -> msg.  Parses text into a 3 key hash: command, param, & data.

    The core syntax is:

      'command:param data'

    The message does not need to contain every field.  It can contain
    essentially any conceivable combination of the three, all the way down to
    a single field (e.g. 'command', ':param', or ' data'), or even a totally
    empty string.
    """

    s = text.find(" ")

    if s != -1:
      msg = { "command": text[:s], "data": text[s:].lstrip(" ") }
    else:
      msg = { "command": text, "data": "" }

    c = msg["command"].find(":")

    if c != -1:
      msg["param"] = msg["command"][c:].lstrip(":")
      msg["command"] = msg["command"][:c]
    else:
      msg["param"] = ""

    return msg



####################  demo  #######################
# yes, this is as ugly as possible :)

if __name__ == "__main__":
  from Thump.Configuration import addresses, ports
  import socket, sys; sys.stderr = sys.stdout
  q = SocketCommandCollector(("tcp", (addresses["queue"], ports["queueToPlayer"])))
  print "listening on port", ports["queueToPlayer"]
  while 1: print q.pop()

