from os import getpid
from sys import argv
from time import asctime

from Configuration import filenames

## specific to thump ##

class PlayerPidFile(PidFile):
  def __init__(self, mode="rw"):
    PidFile.__init__(self, filenames["playerpidfile"], mode)

class QueuePidFile(PidFile):
  def __init__(self, mode="rw"):
    PidFile.__init__(self, filenames["queuepidfile"], mode)

class Playlist(File):
  "Contains path to a file on each line"

  def __init__(self, mode="rw"):
    File.__init__(self, filenames["playlist"], mode)

  def getSong(self):
    return self.readline().strip()
  def getAllSongs(self):
    return [ line.strip() for line in self.readlines() ]

class AccessFile(File):

  # HELLO

  def __init__(self, mode="rw"):
    File.__init__(self, filenames["accesslog"], mode)

  def __dict_from_line(self, line):
    items = [ item.strip() for item in line.split(":@@:") ]
    return { "time": items[0], "song": items[1] }

  def logAccess(self, song):
    self.write("%s :@@: %s\n" % (asctime(), song))
  
  def getSong(self):
    return self.__dict_from_line(self.readStrippedLine())

  def getAllSongs(self):
    return [ self.__dict_from_line(line) for line in self.readStrippedLines() ]

class ErrorLog(File):

  def __init__(self, mode="a"):
    File.__init__(self, filenames["errorlog"], mode)
  
  def writeError(self, msg):
    self.write("%s %s (%s): %s\n" % (asctime(), argv[0].split("/")[-1], getpid(), msg))
    self.flush()

  def unknownCommand(self, command):
    self.writeError("Received unknown command `%s'" % command)

  def decoderHung(self):
    self.writeError("Decoder is hung")

  def die(self, msg=None):
    if msg: self.writeError(msg)
    self.writeError("Exiting")

class StderrFile(File):

  def __init__(self, mode="w"):
    File.__init__(self, filenames["stderr"], mode)

