#!/usr/bin/env python

class CursesWindow:

  children = dict()
  parent = object()
  window = object()

  def __init__(self, parent, ysize, xsize, ystart, xstart):
    self.maxy, self.maxx = parent.getmaxyx()

    if ysize is None: ysize = self.maxy
    if xsize is None: xsize = self.maxx
    if ysize < 0: ysize = self.maxy-ysize
    if xsize < 0: xsize = self.maxx-xsize

    self.window = parent.subwin(ysize, xsize, ystart, xstart)
    self.parent = parent

  def subWin(self, name, ysize, xsize, ystart, xstart):
    self.children[name] = CursesWindow(self, ysize, xsize, ystart, xstart)
    return self.children[name]

  def strWin(self, name, text, ysize, xsize, ystart, xstart):
    self.children[name] = CursesStrWindow(self, text, ysize, xsize, ystart, xstart)
    return self.children[name]

  def clearAndRefresh(self):
    self.clear()
    self.refresh()

  # wrappers around curses stuff
  def clear(self):
    self.window.clear()

  def refresh(self):
    self.maxy, self.maxx = self.getmaxyx()
    self.window.refresh()
    for child in self.children.keys():
      self.children[child].window.refresh()

  def getmaxyx(self):
    return self.window.getmaxyx()

  def subwin(self, ysize, xsize, ystart, xstart):
    return self.window.subwin(ysize, xsize, ystart, xstart)


class CursesStrWindow(CursesWindow):

  def __init__(self, name, text, ysize, xsize, ystart, xstart):
    CursesWindow.__init__(self, name, ysize, xsize, ystart, xstart)

    self.text = text
    self.addstr()


  def addstr(self, text=None):
    if text != None: self.text=text
    self.window.addstr(0, 0, self.text)

  def clear(self):
    self.clear()
    self.addstr()

  def getline(self, pad=1):
    return self.window.getstr(0,len(self.text)+pad)

